# frozen_string_literal: true

require 'bauk/utils/log'
require 'bauk/gen/configs/files'
require 'fileutils'

module Bauk
  module Gen
    # Class purely used for init
    module Init
      include Bauk::Utils::Log
      def init
        FileUtils.mkdir_p '.bauk/generator'
        init_templates.each do |file,content|
          file = ".bauk/generator/templates/#{name}/#{file}"
          FileUtils.mkdir_p File.dirname(file.to_s)
          File.write(file.to_s, content)
        end
        init_files.each do |file,content|
          FileUtils.mkdir_p File.dirname(file.to_s)
          File.write(file.to_s, content)
        end
        init_config_file
      end

      def init_files
        {}
      end

      def init_templates
        {
          'exampleGenerated/example..erb': <<~FILE
            <%= name %>
          FILE,
          'example_generated..erb': <<~FILE
            Name: <%= name %>
            Description: <%= description %>
          FILE
        }
      end

      def init_config_file
        file = Bauk::Gen::Configs::Files.new.default_files[0]
        if File.exist? file
          log.error "File already exists: #{file}"
        else
          init_config = default_config
          unless init_config[:generators]
            init_config[:generators] = {}
            init_config[:generators][name.to_sym] = {}
          end
          File.write(file, YAML.dump(init_config))
        end
      end
    end
  end
end
